;##################################################################
; HintText
;##################################################################

#include HintText.h

#DEFINE UEM_SETHINTTEXT WM_USER+1
#DEFINE UEM_SHOWHINTTEXT WM_USER+2
#DEFINE UEM_GETHINTTEXT WM_USER+3
#DEFINE UEM_SETHINTCLR WM_USER+4
#DEFINE UEM_SETHINTFONT WM_USER+5

DATA SECTION
	hInstance	HANDLE		?

	szHintText	CHAR	256 DUP (?)

CODE SECTION

START:
	invoke GetModuleHandle, 0
	mov [hInstance],eax
	invoke InitCommonControls
	invoke DialogBoxParam,[hInstance],1000,0,ADDR DlgProc,0
	invoke ExitProcess,0

DlgProc FRAME hwnd,uMsg,wParam,lParam
	LOCAL cbi:COMBOBOXINFO

	cmp D[uMsg],WM_INITDIALOG
	jne >>.WM_COMMAND
		invoke GetDlgItem,[hwnd],1001
		invoke InitHintBox,eax,"Enter your name"
		invoke GetDlgItem,[hwnd],1003
		invoke InitHintBox,eax,"Enter your comment"
		mov eax,TRUE
		ret

	.WM_COMMAND
	cmp D[uMsg],WM_COMMAND
	jne >.WM_CLOSE
		movzx edx,W[wParam]
		cmp edx,1002
		jne >>.EXIT
		invoke SendDlgItemMessage,[hwnd],1001,UEM_GETHINTTEXT,offset szHintText,256
		invoke SendDlgItemMessage,[hwnd],1001,UEM_SETHINTTEXT,"Changed the text",0
		invoke SendDlgItemMessage,[hwnd],1001,UEM_SETHINTCLR,0,0xFF
		jmp >.EXIT

	.WM_CLOSE
	cmp D[uMsg],WM_CLOSE
	jne >.DEFPROC
		INVOKE EndDialog,[hwnd],0

	.DEFPROC
		mov eax,FALSE
		ret

	.EXIT

	mov eax, TRUE
	ret
ENDF

InitHintBox FRAME hEditControl, pDefHintText
	uses edi
	LOCAL ClassName[256]:%CHAR

	// Must be an edit control
	invoke GetClassName,[hEditControl],offset ClassName,256
	invoke lstrcmpi,offset ClassName,"edit"
	test eax,eax
	jnz >>.ERROR

	// Sets up the subclass and sets up the default data
	// Create a buffer to hold the hint text
	invoke GlobalAlloc,GMEM_FIXED,128
	mov edi,eax
	// By default the text is shown
	// If the edit has focus when the parent is shown
	// the text will NOT display
	mov D[edi],1
	// Default font
	invoke GetStockObject,DEFAULT_GUI_FONT
	mov [edi+4],eax
	// Default color
	invoke GetSysColor,COLOR_GRAYTEXT
	mov [edi+8],eax
	// Default background color
	invoke GetSysColor,COLOR_WINDOW
	mov [edi+12],eax
	mov eax,edi
	add eax,64
	invoke lstrcpyn,eax,[pDefHintText],64
	// 64 bytes for the flags, data and future expansion + 64 bytes for the text the rest is unused
	invoke SetWindowSubclass,[hEditControl],offset EditSubClass,0,edi
	xor eax,eax
	ret

	.ERROR
	invoke SetLastError,ERROR_INVALID_PARAMETER
	xor eax,eax
	dec eax
	ret
endf

EditSubClass FRAME hwnd, uMsg, wParam, lParam, uIdSubclass, dwRefData
	uses edi,esi,ebx
	LOCAL ps:PAINTSTRUCT
	LOCAL rc:RECT

	// Windows messages

	.WM_PAINT
	cmp D[uMsg],WM_PAINT
	jne >>.WM_SETFOCUS
		// The hint text is only displayed if the show bit is set
		mov edi,[dwRefData]
		test edi,edi
		jz >>.DEFPROC
		mov eax,[edi]
		and eax,1
		jz >>.DEFPROC
		
		// The bit is set so draw the text
		invoke BeginPaint,[hwnd],offset ps
			// NOTE that since there is no way to tell what the default colors
			// are in any given system no hard coded colors can be set here.
			// the defaults are derived from GetSysColor in the InitHintBox function

			// Background mode is transparent so the text doesn't obscure the background
			invoke SetBkMode,[ps.hdc],TRANSPARENT
			invoke SetTextColor,[ps.hdc],[edi+8]
			// Get the text for this particular entry
			invoke GetClientRect,[hwnd],offset rc
			invoke CreateSolidBrush,[edi+12]
			mov ebx,eax
			invoke FillRect,[ps.hdc],offset rc,ebx
			invoke DeleteObject,ebx
			// Set the font
			invoke SelectObject,[ps.hdc],[edi+4]
			// Get the rect to draw the text in, the client rectangle might not match the
			// rectangle the text needs to be displayed in
			invoke SendMessage,[hwnd],EM_GETRECT,0,offset rc
			// Draw the text
			mov eax,edi
			add eax,64
			invoke DrawText,[ps.hdc],eax,-1,offset rc,DT_LEFT+DT_SINGLELINE+DT_VCENTER+DT_CENTER
		invoke EndPaint,[hwnd],offset ps
		xor eax,eax
		ret

	.WM_SETFOCUS
	cmp D[uMsg],WM_SETFOCUS
	jne >.WM_KILLFOCUS
		mov edi,[dwRefData]
		and D[edi],0x0FFFFFFFE
		invoke RedrawWindow,[hwnd],NULL,NULL,RDW_UPDATENOW+RDW_INVALIDATE+RDW_ERASE
		jmp >>.DEFPROC

	.WM_KILLFOCUS
	cmp D[uMsg],WM_KILLFOCUS
	jne >>.UEM_SETHINTFONT
		// If there is text then do not draw the hint text
		invoke SendMessage,[hwnd],WM_GETTEXTLENGTH,0,0
		test eax,eax
		jnz >>.DEFPROC
		mov edi,[dwRefData]
		or D[edi],1
		invoke RedrawWindow,[hwnd],NULL,NULL,RDW_UPDATENOW+RDW_INVALIDATE+RDW_ERASE
		jmp >>.DEFPROC

	// Custom Messages

	.UEM_SETHINTFONT
	cmp D[uMsg],UEM_SETHINTFONT
	jne >.UEM_SETHINTTEXT
		mov eax,[lParam]
		mov edi,[dwRefData]
		mov [edi+4],eax
		xor eax,eax
		ret

	.UEM_SETHINTTEXT
	cmp D[uMsg],UEM_SETHINTTEXT
	jne >.UEM_GETHINTTEXT
		// Sets the hint text
		// skip over the flag data
		mov eax,[dwRefData]
		add eax,64
		invoke lstrcpyn,eax,[wParam],64
		// return the result from lstrcpyn
		ret

	.UEM_GETHINTTEXT
	cmp D[uMsg],UEM_GETHINTTEXT
	jne >.UEM_SHOWHINTTEXT
		// Gets the hint text
		// skip over the flag data
		mov ecx,64
		mov edx,[lParam]
		mov eax,[dwRefData]
		add eax,ecx
		cmp edx,64
		cmovg edx,ecx
		invoke lstrcpyn,[wParam],eax,edx
		// return the result from lstrcpyn
		ret

	.UEM_SHOWHINTTEXT
	cmp D[uMsg],UEM_SHOWHINTTEXT
	jne >.UEM_SETHINTCLR
		// Set the show bit
		mov eax,[dwRefData]
		// Only the least significant bit is used
		and D[lParam],1
		or eax,[lParam]
		ret

	.UEM_SETHINTCLR
	cmp D[uMsg],UEM_SETHINTCLR
	jne >.WM_CLOSE
		// Set the color
		mov eax,[dwRefData]
		mov ecx,[lParam]
		mov [eax+8],ecx
		mov eax,ecx
		ret

	// This is only processed once so stick it at the end
	.WM_CLOSE
	cmp D[uMsg],WM_CLOSE
	jne >.DEFPROC
		mov edi,[dwRefData]
		invoke GlobalFree,edi

	.DEFPROC
	invoke DefSubclassProc, [hwnd], [uMsg], [wParam], [lParam]
	ret
endf
